/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : to7.h
 *  Version    : 1.4
 *  Cr par   : Eric Botcazou 1999
 *  Modifi par: Eric Botcazou 19/12/2000
 *
 *  Module de pilotage de l'mulateur
 */

#ifndef TO7_H
#define TO7_H

#define TO7_VERSION "Thom 1.4"

#define TO7_FRAME_FREQ         50    /* Hz: frquence de trame vido */
#define TO7_CYCLES_PER_FRAME   19968 /* dure exacte en nb de cycles CPU d'une frame */
#define TO7_CPU_FREQ           (TO7_FRAME_FREQ*TO7_CYCLES_PER_FRAME)  /* Hz */
#define TO7_NCOLORS            16    /* nombre de couleurs du TO7-70 */

#define TO7_SCREEN_WIDTH       320  /* largeur de l'cran */
#define TO7_SCREEN_HEIGHT      200  /* hauteur de l'cran */
#define TO7_BORDER_SIZE         16  /* largeur de la bordure de l'cran */

#define TO7_MEMO7_LABEL_LENGTH 25

#define TO7_SHIFT_FLAG             (1<<0)
#define TO7_CTRL_FLAG              (1<<1)
#define TO7_CAPSLOCK_FLAG          (1<<2)
#define TO7_MAX_FLAG               3

enum {
    TO7_ERROR,
    TO7_OK,
    TO7_READ_ONLY,
    TO7_READ_WRITE,
};

/* fonctions importables requises */
extern void (*to7_DrawGPL)(int, int, int);
extern void (*to7_PutSoundByte)(unsigned long long int, unsigned char);

/* fonctions importables optionnelles */
extern void (*to7_SetBorderColor)(int);
extern void (*to7_SetNoCapsLed)(int);

/* variables exportables */
extern char to7_error_msg[];

/* fonctions exportables */
extern int   to7_Init(void);
extern void  to7_Exit(void);
extern int   to7_LoadMemo7(const char []);
extern char* to7_GetMemo7Label(void);
extern void  to7_Reset(void);
extern void  to7_ColdReset(void);
extern void  to7_InputReset(int, int);
extern void  to7_DoFrame(void);
extern void  to7_QueryColor(int, int *, int *, int *);
extern void  to7_HandleKeyPress(int, int);
extern void  to7_HandleMouseMotion(int, int);
extern void  to7_HandleMouseClick(int, int);
extern  int  to7_LoadK7(const char []);
extern  int  to7_SetK7Mode(int);
extern void  to7_RewindK7(void);

#endif
