/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : djgpp/gui.c
 *  Version    : 1.4
 *  Cr par   : Eric Botcazou 1999
 *  Modifi par: Eric Botcazou 15/12/2000
 *
 *  Panneau de contrle de l'mulateur
 */

#include <string.h>
#include <dir.h>
#include <allegro.h>
#include "main.h"
#include "to7.h"
#include "djgpp/sound.h"

#define FG_COLOR       3
#define BG_COLOR       4
#define BG_ENTRY_COLOR 5

static char title[35], mesg[35], quest[35];

static DIALOG mesgdial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc, 10,   10,    0,   50,     0,     0,   0,    0,       0,    0,    NULL },
{ d_ctext_proc,      10,   20,    0,    0,     0,     0,   0,    0,       0,    0,    mesg },
{ d_button_proc,     30,   40,   32,   16,     0,     0, 'o',    D_EXIT,  0,    0,    "&OK" },
{ NULL,               0,    0,    0,    0,     0,     0,   0,    0,       0,    0,    NULL }
};

#define BUTTON_FIX 6

static void PopupMessage(const char message[])
{
    mesgdial[0].w=strlen(message)*8+16;
    mesgdial[1].x=mesgdial[0].x+mesgdial[0].w/2;
    mesgdial[2].x=mesgdial[1].x-(mesgdial[2].w+BUTTON_FIX)/2;
    
    strcpy(mesg, message);

    centre_dialog(mesgdial);

    popup_dialog(mesgdial, 2);
}


static DIALOG questdial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc, 10,   10,    0,   50,     0,     0,   0,    0,       0,    0,    NULL },
{ d_ctext_proc,      10,   20,    0,    0,     0,     0,   0,    0,       0,    0,    quest },
{ d_button_proc,     30,   40,   32,   16,     0,     0, 'o',    D_EXIT,  0,    0,    "&Oui" },
{ d_button_proc,    130,   40,   32,   16,     0,     0, 'n',    D_EXIT,  0,    0,    "&Non" },
{ NULL,               0,    0,    0,    0,     0,     0,   0,    0,       0,    0,    NULL }
};

#define FOCUS_YES  2
#define FOCUS_NO   3

static int PopupQuestion(const char question[], int focus)
{
    int esp;
    
    questdial[0].w=strlen(question)*8+16;
    questdial[1].x=questdial[0].x+questdial[0].w/2;

    esp=(questdial[0].w-2*(questdial[2].w+BUTTON_FIX))/3;
    questdial[2].x=questdial[0].x+esp;
    questdial[3].x=questdial[2].x+(questdial[2].w+BUTTON_FIX)+esp;
    
    strcpy(quest, question);

    centre_dialog(questdial);
    
    return (popup_dialog(questdial, focus) == 2 ? TRUE : FALSE);
}


#define LABEL_LENGTH     20
static char k7_label[LABEL_LENGTH+1] = "aucune cassette";

static DIALOG k7dial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,  280,  180,     0,     0,    0,    0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,    0,    0,     0,     0,    0,    0,       0,    0,    title },
{ d_ctext_proc,      160,  30,    0,    0,     0,     0,    0,    0,       0,    0,    "Lecteur de cassettes" },
{ d_text_proc,        30,  54,    0,    0,     0,     0,    0,    0,       0,    0,    "k7" },
{ d_textbox_proc,     47,  50,  167,   16,     0,     0,    0,    0,       0,    0,    k7_label },
{ d_button_proc,     220,  50,   30,   16,     0,     0,  'k',    D_EXIT,  0,    0,    "..." },
{ d_check_proc,      260,  50,   15,   15,     0,     0,    0,    D_EXIT|D_SELECTED,0,1,"" },
{ d_text_proc,       255,  40,    0,    0,     0,     0,   0,     0,       0,    0,    "prot." },
{ d_button_proc,      30,  74,  250,   16,     0,     0,  'r',    D_EXIT,  0,    0,    "&Rembobiner la cassette" },
{ d_button_proc,      30, 170,  126,   16,     0,     0,  'o',    D_EXIT,  0,    0,    "&OK" },
{ NULL,                0,   0,    0,    0,     0,     0,    0,    0,       0,    0,    NULL }
};

static void menuk7(void)
{
    static int first=1;
    static char filename[FILENAME_LENGTH];
  
    /* la premire fois on tente d'ouvrir le rpertoire par dfaut */
    if (first)
    {
    	if (file_exists("./k7", FA_DIREC, NULL))
	    strcpy(filename,"./k7/");

        centre_dialog(k7dial);

	first=0;
    }

    clear_keybuf();

    while (TRUE)
    {
        int ret=popup_dialog(k7dial, 9);
        
        switch (ret)
        {
            case 5:
                if (file_select("Choisissez votre cassette:",filename,"k7"))
                {
                    int ret=to7_LoadK7(filename);

                    if (ret == TO7_ERROR)
                        PopupMessage(to7_error_msg);
                    else
                    {
                        memset (k7_label, 0, LABEL_LENGTH);
                        strncpy(k7_label, get_filename(filename), LABEL_LENGTH);

                        if ((ret==TO7_READ_ONLY) && !(k7dial[6].d2))
                        {
                            PopupMessage("Attention: criture impossible.");
                            k7dial[6].flags|=D_SELECTED;
                            k7dial[6].d2=1;
                        }
                    }
                }
                break;

            case 6:
                if (k7dial[6].d2)
                {
                    if (to7_SetK7Mode(TO7_READ_WRITE)==TO7_READ_ONLY)
                    {
                        PopupMessage("Ecriture impossible sur ce support.");
                        k7dial[6].flags|=D_SELECTED;
                        k7dial[6].d2=1;
                    }
                    else
                    {
                        k7dial[6].flags&=~D_SELECTED;
                        k7dial[6].d2=0;
                    }
                }
                else
                {
                    to7_SetK7Mode(TO7_READ_ONLY);
                    k7dial[6].flags|=D_SELECTED;
                    k7dial[6].d2=1;
                }
                break;

            case 8:
                to7_RewindK7();
                break;

            case 9:
                return;
        }
    }
}


static char m7_label[TO7_MEMO7_LABEL_LENGTH+1] = "aucune cartouche";

static DIALOG m7dial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,  280,  180,     0,     0,    0,    0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,    0,    0,     0,     0,    0,    0,       0,    0,    title },
{ d_ctext_proc,      160,  30,    0,    0,     0,     0,    0,    0,       0,    0,    "Lecteur de cartouches" },
{ d_text_proc,        30,  54,    0,    0,     0,     0,    0,    0,       0,    0,    "m7" },
{ d_textbox_proc,     47,  50,  208,   16,     0,     0,    0,    0,       0,    0,    m7_label },
{ d_button_proc,     260,  50,   30,   16,     0,     0,  'm',    D_EXIT,  0,    0,    "..." },
{ d_button_proc,      30, 170,  126,   16,     0,     0,  'o',    D_EXIT,  0,    0,    "&OK" },
{ NULL,                0,   0,    0,    0,     0,     0,    0,    0,       0,    0,    NULL }
};

static void menumemo7(void)
{
    static int first=1;
    static char filename[FILENAME_LENGTH];

    /* la premire fois on tente d'ouvrir le rpertoire par dfaut */
    if (first)
    {
	if (file_exists("./memo7", FA_DIREC, NULL))
	    strcpy(filename,"./memo7/");
	
        centre_dialog(m7dial);

	first=0;
    }

    clear_keybuf();

    while (TRUE)
        switch (popup_dialog(m7dial, 6))
        {
            case 5:
                if (file_select("Choisissez votre cartouche:",filename,"m7"))
                {
                    if (to7_LoadMemo7(filename))
                    {
                        strncpy(m7_label, to7_GetMemo7Label(), TO7_MEMO7_LABEL_LENGTH);
                        thom.command=COLD_RESET;
                    }
                    else
                        PopupMessage(to7_error_msg);
                }
                break;

            case 6:
                return;
        }
}


static DIALOG commdial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)    (fg)   (bg)   (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,   280,  180,     0,     0,      0,   0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,     0,    0,     0,     0,      0,   0,       0,    0,    title },
{ d_ctext_proc,      160,  30,     0,    0,     0,     0,      0,   0,       0,    0,    "Commandes et Rglages" },
{ d_button_proc,      30,  50,   260,   16,     0,     0,    'r',   D_EXIT,  0,    0,    "&Rinitialiser le TO7-70" },
{ d_button_proc,      30,  74,   260,   16,     0,     0,    'f',   D_EXIT,  0,    0,    "&Redmarrer  &froid le TO7-70" },
{ d_text_proc,        30, 100,     0,    0,     0,     0,      0,   0,       0,    0,    "Vitesse:" },
{ d_radio_proc,      105, 100,   126,    8,     0,     0,    'e',   0,       1,    0,    "&exacte" },
{ d_radio_proc,      105, 114,   126,    8,     0,     0,    'm',   0,       1,    0,    "&maximale" },
{ d_text_proc,        30, 138,     0,    0,     0,     0,      0,   0,       0,    0,    "Volume &sonore:" },
{ d_text_proc,       150, 152,     0,    0,     0,     0,      0,   0,       0,    0,    "bas         haut" },
{ d_slider_proc,     160, 135,   100,   15,     0,     0,    's',   0,     254,    0,    NULL},
{ d_button_proc,      30, 170,   126,   16,     0,     0,    'o',   D_EXIT,  0,    0,    "&OK" },
{ NULL,                0,   0,     0,    0,     0,     0,      0,   0,       0,    0,    NULL }
};

static void menucomm(void)
{
    if (thom.exact_speed)
    {
        commdial[6].flags=D_SELECTED;
        commdial[7].flags=0;
    }
    else
    {
        commdial[6].flags=0;
        commdial[7].flags=D_SELECTED;
    }

    if (thom.sound_enabled && thom.exact_speed)
        commdial[10].d2=GetVolume()-1;

    /* we never read the key buffer, we have to clear it to avoid problems */
    clear_keybuf();
    centre_dialog(commdial);

    switch (popup_dialog(commdial, 11))
    {
        case 3:
            thom.command=RESET;
            break;

        case 4:
            thom.command=COLD_RESET;
            break;
    }

    if (thom.sound_enabled && thom.exact_speed)
        SetVolume(commdial[10].d2+1);

    thom.exact_speed=(commdial[6].flags&D_SELECTED ? TRUE : FALSE);
}


/*
 * ControlPanel: affiche le panneau de contrle de l'mulateur
 */

static DIALOG controldial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg) (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,  280,  180,     0,     0,   0,    0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,    0,    0,     0,     0,   0,    0,       0,    0,    title },
{ d_ctext_proc,      160,  30,    0,    0,     0,     0,   0,    0,       0,    0,    "Sylvain Huet - Eric Botcazou" },
{ d_button_proc,     30,   60,  260,   16,     0,     0,   'c',  D_EXIT,  0,    0,    "&Commandes et Rglages..." },
{ d_button_proc,     30,   84,  260,   16,     0,     0,   't',  D_EXIT,  0,    0,    "Lecteur de car&touches..." },
{ d_button_proc,     30,  108,  260,   16,     0,     0,   's',  D_EXIT,  0,    0,    "Lecteur de ca&ssettes..." },
{ d_button_proc,     30,  170,  126,   16,     0,     0,   'o',  D_EXIT,  0,    0,    "&OK" },
{ d_button_proc,    164,  170,  126,   16,     0,     0,   'q',  D_EXIT,  0,    0,    "&Quitter" },
{ NULL,               0,    0,    0,    0,     0,     0,   0,    0,       0,    0,    NULL }
};

void ControlPanel(void)
{
    /* we never read the key buffer, we have to clear it to avoid problems */
    clear_keybuf();
    centre_dialog(controldial);

    while (TRUE)
        switch (popup_dialog(controldial, 6))
        {
            case 3:
                menucomm();
                break;

            case 4:
                menumemo7();
                break;

            case 5:
                menuk7();
                break;

            case 6:
                return;

            case 7:
                if (PopupQuestion("Voulez-vous vraiment quitter ?", FOCUS_NO))
                {
                    thom.command=QUIT;
                    return;
                }
        }
}


void InitGUI(char version_name[])
{
    strcpy(title, version_name);

    if (!thom.sound_enabled)
        commdial[8].flags=commdial[9].flags=commdial[10].flags=D_DISABLED;

    if (strlen(to7_GetMemo7Label()))
        strncpy(m7_label, to7_GetMemo7Label(), TO7_MEMO7_LABEL_LENGTH);

    set_dialog_color( mesgdial, FG_COLOR, BG_COLOR);
    set_dialog_color(questdial, FG_COLOR, BG_COLOR);
    
    set_dialog_color(k7dial, FG_COLOR, BG_COLOR);
    k7dial[4].bg = BG_ENTRY_COLOR;

    set_dialog_color(m7dial, FG_COLOR, BG_COLOR);
    m7dial[4].bg = BG_ENTRY_COLOR;
    
    set_dialog_color(commdial, FG_COLOR, BG_COLOR);
    set_dialog_color(controldial, FG_COLOR, BG_COLOR);

    /* Allegro GUI objects */
    gui_fg_color = FG_COLOR;
    gui_bg_color = BG_COLOR;
    gui_mg_color = BG_COLOR;
}
