/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : djgpp/mouseint.c
 *  Version    : 1.4
 *  Cr par   : Eric Botcazou 1999
 *  Modifi par: Eric Botcazou 17/12/2000
 *
 *  Gestion du crayon optique du TO7-70
 */

#include <allegro.h>
#include "to7.h"
#include "djgpp/graphic.h"

static BITMAP *pen_pointer;
static char pen_pointer_data[64]={ 0,0,1,1,1,0,0,0,
                                    0,0,1,2,1,0,0,0,
                                    1,1,1,2,1,1,1,0,
                                    1,2,2,2,2,2,1,0,
                                    1,1,1,2,1,1,1,0,
                                    0,0,1,2,1,0,0,0,
                                    0,0,1,1,1,0,0,0,
                                    0,0,0,0,0,0,0,0 };

/*
 * LightpenCallback: routine de gestion du crayon optique du TO7-70 appele par
 *                   l'interruption matrielle souris INT 0x33 via Allegro
 */

static void LightpenCallback(int flags)
{
    static int pointer_on=1, xpos, ypos;

    if (flags&MOUSE_FLAG_LEFT_UP)
        to7_HandleMouseClick(1, TRUE);
    if (flags&MOUSE_FLAG_LEFT_DOWN)
        to7_HandleMouseClick(1, FALSE);

    if (flags&(MOUSE_FLAG_MOVE|MOUSE_FLAG_RIGHT_DOWN))
    {
        if (pointer_on)
            RetraceScreen( xpos-pen_pointer->w/2+1, ypos-pen_pointer->h/2+1,
                           pen_pointer->w         , pen_pointer->h );
                           
        to7_HandleMouseMotion(mouse_x, mouse_y);

        if (flags&MOUSE_FLAG_RIGHT_DOWN)
            pointer_on^=1;

        if (pointer_on)
        {
            xpos = mouse_x;
            ypos = mouse_y;
            draw_sprite(screen, pen_pointer, xpos-pen_pointer->w/2+1,
                                             ypos-pen_pointer->h/2+1 );
        }
    }
}

static END_OF_FUNCTION(LightpenCallback)


/*
 * InstallPointer: installe le crayon optique du TO7-70
 */

void InstallPointer(void)
{
    mouse_callback=LightpenCallback;
}


/*
 * ShutDownPointer: dsactive crayon optique du TO7-70
 */

void ShutDownPointer(void)
{
    mouse_callback=NULL;
}


/*
 * InitMouse: initialise le module crayon optique
 */

void InitPointer(void)
{
    register int i;

    /* cration du pointeur crayon optique */
    pen_pointer=create_bitmap(8,8);

    for (i=0; i<64; i++)
        switch (pen_pointer_data[i])
        {
            case 1:
                pen_pointer->line[i/8][i&7] = 1;
                break;
            case 2:
                pen_pointer->line[i/8][i&7] = 2;
                break;
            default:
                pen_pointer->line[i/8][i&7] = 0;
        }

    install_mouse();

    lock_bitmap(pen_pointer);
    LOCK_FUNCTION(LightpenCallback);
}
