/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : djgpp/sound.c
 *  Version    : 1.4
 *  Cr par   : Eric Botcazou Avril 1999
 *  Modifi par: Eric Botcazou 16/12/2000
 *
 *  Gestion de l'mulation sonore du TO7-70
 */

#include <stdio.h>
#include <string.h>
#include <allegro.h>
#include "main.h"
#include "to7.h"


#define SOUND_FREQ  25600  /* Hz: frquence d'chantillonage */
#define SOUND_BUFFER_SIZE (SOUND_FREQ/TO7_FRAME_FREQ)

static AUDIOSTREAM *stream;
static unsigned char sound_buffer[SOUND_BUFFER_SIZE];
static int last_index;
static unsigned char last_data;


void StartSound(void)
{
    static int first=1;

    if (first)
    {
        stream=play_audio_stream(SOUND_BUFFER_SIZE,8,FALSE,SOUND_FREQ,128,128);
        first=0;
    }
    else
        voice_start(stream->voice);
}


void StopSound(void)
{
    voice_stop(stream->voice);
}


int GetVolume(void)
{
    return voice_get_volume(stream->voice);
}


void SetVolume(int val)
{
    voice_set_volume(stream->voice,val);
}


static void PutSoundByte(unsigned long long int clock, unsigned char data)
{
    register int i;
    int index=(clock%TO7_CYCLES_PER_FRAME)*SOUND_FREQ/TO7_CPU_FREQ;

    /* Dans le cas o le nombre de cycles xcuts pendant une frame dpasse la valeur
       thorique, on bloque l'index  sa valeur maximale */
    if (index < last_index)
	index=SOUND_BUFFER_SIZE;

    for (i=last_index; i<index; i++)
        sound_buffer[i]=last_data;

    last_index=index;
    last_data=data;
}


void PlaySoundBuffer(void)
{
    register int i;
    char *buffer_ptr;

    /* on remplit la fin du buffer avec le dernier byte dpos */
    for (i=last_index; i<SOUND_BUFFER_SIZE; i++)
        sound_buffer[i]=last_data;

    last_index=0;

    while ( (buffer_ptr=get_audio_stream_buffer(stream)) == NULL)
        ;

    memcpy(buffer_ptr, sound_buffer, SOUND_BUFFER_SIZE);
    
    free_audio_stream_buffer(stream);
}


static void voice_get_position_callback(void)
{
    if (voice_get_position(stream->voice))
        thom.sound_enabled=TRUE;
}


void InitSound(void)
{
    to7_PutSoundByte=PutSoundByte;

    if (!thom.sound_enabled)
        return;

    thom.sound_enabled=FALSE;

    printf("Initialisation du son...");

    if (!install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL))
    {
        /* test de fonctionnement du streaming */
        stream=play_audio_stream(SOUND_BUFFER_SIZE,8,FALSE,SOUND_FREQ,128,128);
        rest_callback(100, voice_get_position_callback);  /* 100 ms */
        stop_audio_stream(stream);
    }

    if (thom.sound_enabled)
    {
        printf("ok\n");
#ifdef DEBUG
        printf("sound buffer size: %d bytes\n",SOUND_BUFFER_SIZE);
#endif
    }
    else
        printf("chec\n");
}
