/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : k7.c
 *  Version    : 1.4
 *  Cr par   : Eric Avril 1999
 *  Modifi par: Eric Botcazou 15/12/2000
 *
 *  Gestion des cassettes du TO7-70
 */

#include <stdio.h>
#include <string.h>
#include "intern/hardware.h"
#include "to7.h"

static FILE *k7;
static char k7name[FILENAME_LENGTH];
static int k7mode;
static enum {
    READ,
    WRITE
} current_op;


/*
 * DoLoadK7: ouvre le fichier k7 et retourne le mode d'ouverture
 */

static int DoLoadK7(const char filename[], int mode)
{
    if (k7)
        fclose(k7);

    if (mode==TO7_READ_WRITE)
    {
     	if ( (k7=fopen(filename, "rb+")) )
	    return mode;
	else
	    mode=TO7_READ_ONLY;
    }

    if ( (k7=fopen(filename, "rb")) )
        return mode;
    else
        return TO7_ERROR;
}

    
/*
 * DoK7Stuff: mule le contrleur du lecteur de cassettes
 */

void DoK7Stuff(int *br, int *cc)
{
    switch (LOAD_BYTE(0x6029)&0x1F)
    {
        case 1:
            STORE_BYTE(0x602A, 1);
            *cc&=0xfe;
	    break;

        case 2:
	    if ((k7) && !feof(k7)) 
	    {
	        if (current_op==WRITE)
		{
		    fflush(k7); /* pour se conformer  l'ANSI C */
		    current_op=READ;
		}

                *br=fgetc(k7)&0xFF;
		*cc&=0xfe;
	    }
	    else
	    {
                STORE_BYTE(0x602A, 0x80);
		*cc|=1;
	    }
            break;

        case 4:
            if (k7 && (k7mode==TO7_READ_WRITE))
	    {
	        if (LOAD_BYTE(0x602A) != 4)
		{
                    register int i;

		    if (current_op==READ)
		    {
		        fflush(k7); /* pour se conformer  l'ANSI C */
			current_op=WRITE;
		    }

		    for (i=0;i<10;i++)
                        fputc(*br,k7);

                    STORE_BYTE(0x602A, 4);
		}
		*cc&=0xfe;
	    }
	    else
	    {
                STORE_BYTE(0x602A, 0x80);
		*cc|=1;
	    }
            break;

        case 8:
            if ((k7) && (k7mode==TO7_READ_WRITE))
	    {
	        if (current_op==READ)
		{
		    fflush(k7); /* pour se conformer  l'ANSI C */
		    current_op=WRITE;
		}

                fputc(*br,k7);
		*cc&=0xfe;
	    }
	    else
	    {
                STORE_BYTE(0x602A, 0x80);
		*cc|=1;
	    }
            break;

        default:
            STORE_BYTE(0x602A, 0x10);
	    *cc&=0xfe;
    }
}


void InitK7(void)
{
    /* appel routine de gestion K7 */
    mem.mon.bank[0][0x107C]=0x02;
    mem.mon.bank[0][0x107D]=0x39;

    k7=NULL;    
    k7mode=TO7_READ_ONLY;
    current_op=READ;
}


/*
 * partie publique
 */

int to7_LoadK7(const char filename[])
{
    int ret=DoLoadK7(filename, k7mode);

    if (ret != TO7_ERROR)
    {
        strcpy(k7name, filename);
        k7mode=ret;
    }

    return ret;
}


int to7_SetK7Mode(int mode)
{
    if (k7mode == mode)
        return k7mode;
    
    if (k7)
    {
        int ret=DoLoadK7(k7name, mode);

        if (ret != TO7_ERROR)
            k7mode=ret;

        return ret;
    }
    else
    {    k7mode=mode;
         return k7mode;
    }
}


void to7_RewindK7(void)
{
    if (k7)
        fseek(k7,0,SEEK_SET);
}
