/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : linux/graphic.c
 *  Version    : 1.4
 *  Cr par   : Eric Botcazou juillet 1999
 *  Modifi par: Eric Botcazou 27/08/2000
 *
 *  Gestion de l'affichage du TO7-70
 */
 
#include <stdio.h>
#include <stdlib.h> 
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XShm.h>
#include "to7.h" 
#include "linux/display.h"

int need_border_refresh;

#define CELL_COLS  40
#define CELL_ROWS  25

static int modified_cell[CELL_COLS+1][CELL_ROWS];
static XVisualInfo visualinfo;
static Colormap colormap;
static GC gc;
static XColor xcolor[TO7_NCOLORS];
static XImage *buffer;
static int border_color;


/*
 * routines d'affichage
 */

static void (*PutPixel)(XImage *, int, int, unsigned long);

static void PutPixel1(XImage *buffer, int x, int y, unsigned long pixel)
{
    XPutPixel(buffer, x, y, pixel);
}

static void PutPixel2(XImage *buffer, int x, int y, unsigned long pixel)
{
    x<<=1;
    y<<=1;

    XPutPixel(buffer, x,   y,   pixel);
    XPutPixel(buffer, x+1, y,   pixel);
    XPutPixel(buffer, x,   y+1, pixel);
    XPutPixel(buffer, x+1, y+1, pixel);
}


/*
 * RefreshBorder: met  jour la couleur du pourtour de l'cran
 */

void RefreshBorder(void)
{
    XSetWindowBackground(display, main_win, xcolor[8-((border_color&4)<<1)+(border_color>>4)].pixel);
    XClearWindow(display, main_win);
    need_border_refresh=False;
}


/*
 * SetBorderColor: fixe la couleur du pourtour de l'cran
 */

void SetBorderColor(int color)
{
    if (border_color != color)
    {
        border_color=color;
        need_border_refresh=True;
    }
}


/*
 * DrawByte: affiche un octet de VRAM
 */

static void DrawGPL(int addr, int pt, int col)
{
    register int i;
    unsigned int x,y,c1,c2;

    modified_cell[addr%CELL_COLS][addr/(CELL_COLS*8)]=True;

    c1=((col>>3)&7)+(((~col)&0x40)>>3);
    c2=(col&7)+(((~col)&0x80)>>4);

    x=(addr%CELL_COLS)*8;
    y=addr/CELL_COLS;
  	
    for (i=0; i<8; i++)
	PutPixel(buffer, x+i, y, xcolor[pt&(0x80>>i) ? c1 : c2].pixel);
}


/*
 * InitGraphic: slectionne le visual, met en place la palette de couleurs et
 *              initialise le mcanisme de bufferisation (dirty rectangles)
 */

void InitGraphic(void)
{
    register int i,j;
    int screen_w=SCREEN_WIDTH*screen_size;
    int screen_h=SCREEN_HEIGHT*screen_size;

    /* Recherche et slection du visual */ 
    if (!XMatchVisualInfo(display, screen, 16, TrueColor, &visualinfo))
	if (!XMatchVisualInfo(display, screen, 24, TrueColor, &visualinfo))
	    if (!XMatchVisualInfo(display, screen, 32, TrueColor, &visualinfo))
		if (!XMatchVisualInfo(display, screen, 8, PseudoColor, &visualinfo))
		{
		    fprintf(stderr,"%s: aucun visual disponible.\n",PROG_NAME);
		    exit(EXIT_FAILURE);
		}

    /* Initialisation de la palette de couleurs */
    colormap=DefaultColormap(display, screen);
    gc=DefaultGC(display, screen);

    /* rcupration de la palette du TO7-70 */
    for (i=0; i<TO7_NCOLORS; i++)
    {
        int red, green, blue;

        to7_QueryColor(i, &red, &green, &blue);

	xcolor[i].red   = 0x101*red;
	xcolor[i].green = 0x101*green;
	xcolor[i].blue  = 0x101*blue;
    }

    if (visualinfo.class==TrueColor)
        for (i=0;i<TO7_NCOLORS;i++)
	    XAllocColor(display, colormap, &xcolor[i]);
    else if (visualinfo.class == PseudoColor)
    {
	long int pixels[TO7_NCOLORS];

	if (!XAllocColorCells(display, colormap, False, 0, 0, pixels, TO7_NCOLORS))
	{
	    colormap=XCreateColormap(display, main_win, visualinfo.visual, AllocNone);

	    if (colormap == DefaultColormap(display, screen))
	    {
		fprintf(stderr,"%s: palette de couleurs immuable.\n",PROG_NAME);
		exit(EXIT_FAILURE);
	    }

	    XSetWindowColormap(display, main_win, colormap);
	}

	for (i=0; i<TO7_NCOLORS; i++)
	{
	    xcolor[i].pixel=pixels[i];
	    xcolor[i].flags=DoRed|DoGreen|DoBlue;
	    XStoreColor(display, colormap, &xcolor[i]);
	}
    }
	
    /* Cration du buffer d'affichage */
    if (mit_shm_enabled)
    {
	XShmSegmentInfo *shminfo = malloc(sizeof(XShmSegmentInfo));

        printf("Extension MIT-SHM utilise.\n");
        buffer=XShmCreateImage(display, visualinfo.visual, visualinfo.depth, ZPixmap, 0, shminfo, screen_w, screen_h);

	shminfo->shmid=shmget(IPC_PRIVATE, screen_h*buffer->bytes_per_line, IPC_CREAT | 0777);
	shminfo->shmaddr= buffer->data= (char *) shmat(shminfo->shmid, 0, 0);
	shminfo->readOnly = False;

	XShmAttach (display, shminfo);
	XSync (display, 0);
	
	shmctl (shminfo->shmid, IPC_RMID, 0);
    }
    else
    {
	buffer=XCreateImage(display, visualinfo.visual, visualinfo.depth, ZPixmap, 0, NULL, screen_w, screen_h, 32, 0);
	buffer->data=malloc(screen_h*buffer->bytes_per_line);
    }

#ifdef DEBUG
    printf("buffer:  width=%d, height=%d\n", buffer->width, buffer->height);
    printf("         format=%s\n", buffer->format==XYBitmap ? "XYBitmap" : (buffer->format==XYPixmap ? "XYPixmap" : "ZPixmap"));
    printf("         byte_order=%s\n", buffer->byte_order==LSBFirst ? "LSBFirst" : "MSBFirst");
    printf("         bitmap_unit=%d\n", buffer->bitmap_unit);
    printf("         bitmap_bit_order=%s\n", buffer->bitmap_bit_order==LSBFirst ? "LSBFirst" : "MSBFirst");
    printf("         bitmap_pad=%d\n", buffer->bitmap_pad);
    printf("         depth=%d\n", buffer->depth);
    printf("         bytes_per_lines=%d\n", buffer->bytes_per_line);
    printf("         bits_per_pixel=%d\n",  buffer->bits_per_pixel);
    printf("          red_mask=%lx\n",  buffer->red_mask);
    printf("          green_mask=%lx\n",  buffer->green_mask);
    printf("          blue_mask=%lx\n",  buffer->blue_mask);
#endif

    for (i=0; i<CELL_COLS+1; i++)
        for (j=0; j<CELL_ROWS; j++)
            modified_cell[i][j]=False;

    to7_SetBorderColor=SetBorderColor;
    to7_DrawGPL=DrawGPL;

    if (screen_size==1)
	PutPixel=PutPixel1;
    else if (screen_size==2)
	PutPixel=PutPixel2;
}


/*
 * RedrawScreen: redessine une portion de l'cran
 */
  
inline void RedrawScreen(int x, int y, int w, int h)
{
    if (mit_shm_enabled)
	XShmPutImage(display, screen_win, gc, buffer, x, y, x, y, w, h, 0);
    else
	XPutImage(display, screen_win, gc, buffer, x, y, x, y, w, h);
}


/*
 * RefreshScreen: rafraichit l'cran du TO7-70
 */

void RefreshScreen(void)
{
    register int i,j;
             int cell_start, cell_size=8*screen_size;

    for (j=0; j<CELL_ROWS; j++)
	for (i=0; i<CELL_COLS; i++)
	    if (modified_cell[i][j])
	    {
		cell_start=i;

		while (modified_cell[i][j])
		    modified_cell[i++][j]=False;

		RedrawScreen(cell_start*cell_size, j*cell_size, (i-cell_start)*cell_size, cell_size);
	    }
}
