/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : linux/gui.c
 *  Version    : 1.4
 *  Cr par   : Eric Botcazou juillet 1999
 *  Modifi par: Eric Botcazou 15/12/2000
 *
 *  Interface utilisateur de l'mulateur base sur GTK+
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include "main.h"
#include "to7.h"
#include "linux/display.h"
#include "linux/filentry.h"
#include "linux/graphic.h"
#include "linux/message.h"
#include "linux/question.h"
#include "thomson.xpm"

/* fentre l'interface utilisateur */
static GtkWidget *window;


/*
 * basename: helper pour extraire le nom simple du fichier
 */

static char *basename(char *filename)
{
    while (*filename)
        filename++;

    while (*filename != '/')
        filename--;

    return filename+1;
}


/*
 * retrace_callback: callback de retraage de la fentre principale
 */

static GdkFilterReturn retrace_callback(GdkXEvent *xevent, GdkEvent *event, gpointer unused)
{
    XEvent *ev = (XEvent *) xevent;

    if (ev->type == Expose)
	RedrawScreen(ev->xexpose.x, ev->xexpose.y, ev->xexpose.width, ev->xexpose.height);

    (void) event;
    (void) unused;

    return GDK_FILTER_REMOVE;
}


static void do_exit(GtkWidget *button, int command)
{
    if (command != NONE)
        thom.command=command;

    gtk_widget_hide(window);
    gtk_main_quit();

    (void) button;
}


static void question_exit(GtkWidget *button, gpointer unused)
{
    GtkWidget *quest=question_new("Voulez-vous vraiment quitter l'mulateur ?");
    gtk_signal_connect( GTK_OBJECT(quest), "clicked_yes", GTK_SIGNAL_FUNC(do_exit), (gpointer) QUIT);
    gtk_quit_add_destroy(1, GTK_OBJECT(quest));
    gtk_widget_show(quest);

    (void) button;
    (void) unused;
}


static void toggle_button_true(GtkWidget *button, gpointer unused)
{
    thom.exact_speed=( GTK_TOGGLE_BUTTON(button)->active ? TRUE : FALSE);

    (void) unused;
}


static void read_filent_memo(GtkWidget *filent, gpointer unused)
{
    char *filename=file_entry_get_filename(FILE_ENTRY(filent));

    if (to7_LoadMemo7(filename) == TO7_ERROR)
    {
        GtkWidget *mesg=message_new(to7_error_msg);
	gtk_quit_add_destroy(1, GTK_OBJECT(mesg));
        gtk_widget_show(mesg);
    }
    else
    {
        file_entry_set_entry( FILE_ENTRY(filent), to7_GetMemo7Label());
	thom.command=COLD_RESET;
    }   

    (void) unused;
}


static void read_filent_cass(GtkWidget *filent, GtkToggleButton *button)
{
    int ret=to7_LoadK7( file_entry_get_filename(FILE_ENTRY(filent)) );
   
    if (ret==TO7_ERROR)
    {
        GtkWidget *mesg=message_new(to7_error_msg);
	gtk_quit_add_destroy(1, GTK_OBJECT(mesg));
        gtk_widget_show(mesg);
    }
    else
    {
	file_entry_set_entry( FILE_ENTRY(filent), basename(file_entry_get_filename(FILE_ENTRY(filent))));

	if ((ret==TO7_READ_ONLY) && !button->active)
	{
	    GtkWidget *mesg=message_new("Attention: criture impossible.");
	    gtk_quit_add_destroy(1, GTK_OBJECT(mesg));
	    gtk_widget_show(mesg);
	    gtk_toggle_button_set_active(button, TRUE);
	}
    }
}


static void toggle_button_readonly(GtkWidget *button, gpointer unused)
{
    if ( GTK_TOGGLE_BUTTON(button)->active )
        to7_SetK7Mode(TO7_READ_ONLY);
    else if (to7_SetK7Mode(TO7_READ_WRITE)==TO7_READ_ONLY)
    {
        GtkWidget *mesg=message_new("Eriture impossible sur ce support.");
	gtk_quit_add_destroy(1, GTK_OBJECT(mesg));
	gtk_widget_show(mesg);
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(button), TRUE);
    }

    (void) unused;
}


void InitGUI(const char version_name[])
{
    GtkWidget *vbox_window;
    GtkWidget *hbox_title, *label_title, *pixmapwid1, *pixmapwid2;
    GtkWidget *frame_command, *vbox_command, *button_reset, *button_coldreset;
    GtkWidget *hbox_speed, *label_speed, *button_true, *button_max;
    GtkWidget *frame_memo, *vbox_memo, *filent_memo;
    GtkWidget *frame_cass, *vbox_cass, *filent_cass;
    GtkWidget *hbox_cass,  *button_readonly, *button_rewind;
    GtkWidget *hbox_end, *button_ok, *button_quit;
    GtkStyle  *style;
    GdkPixmap *pixmap;
    GdkBitmap *mask;
    GSList *group_speed;
    char string_title[100];

    printf("Initialisation de l'interface...");
    
    /* fentre d'affichage */
    window=gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_window_set_title( GTK_WINDOW(window), "Thom - Panneau de contrle");
    gtk_window_set_policy(GTK_WINDOW(window), False, False, False);
    gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
    gtk_signal_connect( GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(do_exit), (gpointer) QUIT);
    gtk_widget_realize(window); /* ncessaire pour charger le pixmap */

    /* boite verticale associe  la fentre */
    vbox_window=gtk_vbox_new(FALSE,5);
    gtk_container_set_border_width( GTK_CONTAINER(vbox_window), 5);
    gtk_container_add( GTK_CONTAINER(window), vbox_window);
    gtk_widget_show(vbox_window);

    /* boite horizontale du titre */
    hbox_title=gtk_hbox_new(FALSE,5);
    gtk_box_pack_start( GTK_BOX(vbox_window), hbox_title, TRUE, FALSE, 5);
    gtk_widget_show(hbox_title);
    
    /* chargement du pixmap */
    style=gtk_widget_get_style(window);
    pixmap=gdk_pixmap_create_from_xpm_d(window->window, &mask, &style->bg[GTK_STATE_NORMAL], thomson);

    /* premire instance du pixmap */
    pixmapwid1=gtk_pixmap_new(pixmap,mask);
    gtk_box_pack_start( GTK_BOX(hbox_title), pixmapwid1, TRUE, FALSE, 5);   
    gtk_widget_show(pixmapwid1);

    /* label du titre */
    strcat(strcpy(string_title,version_name),"\nSylvain Huet - Eric Botcazou");
    label_title=gtk_label_new(string_title);
    gtk_box_pack_start( GTK_BOX(hbox_title), label_title, TRUE, FALSE, 5);
    gtk_widget_show(label_title);

    /* deuxime instance du pixmap */
    pixmapwid2=gtk_pixmap_new(pixmap,mask);
    gtk_box_pack_start( GTK_BOX(hbox_title), pixmapwid2, TRUE, FALSE, 5);   
    gtk_widget_show(pixmapwid2);

    /* frame des commandes et rglages */
    frame_command=gtk_frame_new("Commandes et Rglages");
    gtk_box_pack_start( GTK_BOX(vbox_window), frame_command, TRUE, FALSE, 0);
    gtk_widget_show(frame_command);

    /* boite verticale associe  la frame des commandes et rglages */
    vbox_command=gtk_vbox_new(FALSE,5);
    gtk_container_set_border_width( GTK_CONTAINER(vbox_command), 5);
    gtk_container_add( GTK_CONTAINER(frame_command), vbox_command);
    gtk_widget_show(vbox_command);

    /* bouton de rinitialisation */
    button_reset=gtk_button_new_with_label("Rinitialiser le TO7-70");
    gtk_signal_connect( GTK_OBJECT(button_reset), "clicked", GTK_SIGNAL_FUNC(do_exit), (gpointer) RESET);
    gtk_box_pack_start( GTK_BOX(vbox_command), button_reset, TRUE, FALSE, 0);
    gtk_widget_show(button_reset); 

    /* bouton de redmarrage  froid */
    button_coldreset=gtk_button_new_with_label("Redmarrer  froid le TO7-70");
    gtk_signal_connect( GTK_OBJECT(button_coldreset), "clicked", GTK_SIGNAL_FUNC(do_exit), (gpointer) COLD_RESET);
    gtk_box_pack_start( GTK_BOX(vbox_command), button_coldreset, TRUE, FALSE, 0);
    gtk_widget_show(button_coldreset);

    /* boite horizontale de la vitesse */
    hbox_speed=gtk_hbox_new(FALSE,5);
    gtk_box_pack_start( GTK_BOX(vbox_command), hbox_speed, TRUE, FALSE, 0);
    gtk_widget_show(hbox_speed);
    
    /* label de la vitesse */
    label_speed=gtk_label_new("Vitesse de l'mulation:");
    gtk_box_pack_start( GTK_BOX(hbox_speed), label_speed, TRUE, TRUE, 0);
    gtk_widget_show(label_speed);

    /* bouton de vitesse maximale */
    button_max=gtk_radio_button_new_with_label(NULL, "maximale");
    gtk_box_pack_end( GTK_BOX(hbox_speed), button_max, TRUE, TRUE, 0);
    gtk_widget_show(button_max);
    
    /* bouton de vitesse exacte */
    group_speed=gtk_radio_button_group( GTK_RADIO_BUTTON(button_max));
    button_true=gtk_radio_button_new_with_label(group_speed, "exacte");
    gtk_signal_connect( GTK_OBJECT(button_true), "toggled", GTK_SIGNAL_FUNC(toggle_button_true), NULL);
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(button_true), thom.exact_speed ? TRUE : FALSE);
    gtk_box_pack_end( GTK_BOX(hbox_speed), button_true, TRUE, TRUE, 0);
    gtk_widget_show(button_true);
    
    /* frame de la cartouche */
    frame_memo=gtk_frame_new("Lecteur de cartouches");
    gtk_box_pack_start( GTK_BOX(vbox_window), frame_memo, TRUE, FALSE, 0);
    gtk_widget_show(frame_memo);

    /* boite verticale associe  la frame de la cartouche */
    vbox_memo=gtk_vbox_new(FALSE,0);
    gtk_container_set_border_width( GTK_CONTAINER(vbox_memo), 5);
    gtk_container_add( GTK_CONTAINER(frame_memo), vbox_memo);
    gtk_widget_show(vbox_memo);

    /* entre fichier de la cartouche */
    filent_memo=file_entry_new("m7");
    gtk_signal_connect( GTK_OBJECT(filent_memo), "file_selected", GTK_SIGNAL_FUNC(read_filent_memo), NULL);

    if (strlen(to7_GetMemo7Label()))
        file_entry_set_entry( FILE_ENTRY(filent_memo), to7_GetMemo7Label());
    else
    {
	file_entry_set_entry( FILE_ENTRY(filent_memo), "aucune cartouche");

	if (!access("./memo7/", F_OK))
	    file_entry_set_filename( FILE_ENTRY(filent_memo), "./memo7/");
    }

    gtk_box_pack_start( GTK_BOX(vbox_memo), filent_memo, FALSE, FALSE, 0);
    gtk_widget_show(filent_memo);
    
    /* frame de la cassette */
    frame_cass=gtk_frame_new("Lecteur de cassettes");
    gtk_box_pack_start( GTK_BOX(vbox_window), frame_cass, TRUE, FALSE, 0);
    gtk_widget_show(frame_cass);
    
    /* boite verticale associe  la frame de la cassette */
    vbox_cass=gtk_vbox_new(FALSE,5);
    gtk_container_set_border_width( GTK_CONTAINER(vbox_cass), 5);
    gtk_container_add( GTK_CONTAINER(frame_cass), vbox_cass);
    gtk_widget_show(vbox_cass);

    /* boite horizontale du mode de la cassette */
    hbox_cass=gtk_hbox_new(FALSE,5);
    gtk_box_pack_end( GTK_BOX(vbox_cass), hbox_cass, TRUE, TRUE, 0);
    gtk_widget_show(hbox_cass);
    
    /* bouton mode lecture seule*/
    button_readonly=gtk_check_button_new_with_label("Mode lecture seule");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button_readonly), TRUE);
    gtk_signal_connect( GTK_OBJECT(button_readonly), "toggled", GTK_SIGNAL_FUNC(toggle_button_readonly), NULL);
    gtk_box_pack_start( GTK_BOX(hbox_cass), button_readonly, TRUE, TRUE, 0);
    gtk_widget_show(button_readonly);
    
    /* bouton rembobinage */
    button_rewind=gtk_button_new_with_label("Rembobiner la cassette");
    gtk_signal_connect( GTK_OBJECT(button_rewind), "clicked", GTK_SIGNAL_FUNC(to7_RewindK7), NULL);
    gtk_box_pack_start( GTK_BOX(hbox_cass), button_rewind, TRUE, TRUE, 0);
    gtk_widget_show(button_rewind);
    
    /* entre fichier de la cassette */
    filent_cass=file_entry_new("k7");
    gtk_signal_connect( GTK_OBJECT(filent_cass), "file_selected", GTK_SIGNAL_FUNC(read_filent_cass), (gpointer) button_readonly);

    file_entry_set_entry( FILE_ENTRY(filent_cass), "aucune cassette");

    if (!access("./k7/", F_OK))
	file_entry_set_filename( FILE_ENTRY(filent_cass), "./k7/");

    gtk_box_pack_start( GTK_BOX(vbox_cass), filent_cass, TRUE, TRUE, 0);
    gtk_widget_show(filent_cass);

    /* boite horizontale des boutons de sortie */
    hbox_end=gtk_hbutton_box_new();
    gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_SPREAD);
    gtk_box_pack_start( GTK_BOX(vbox_window), hbox_end, TRUE, FALSE, 0);
    gtk_widget_show(hbox_end);
    
    /* bouton retour */
    button_ok=gtk_button_new_with_label("OK");
    gtk_signal_connect( GTK_OBJECT(button_ok), "clicked", GTK_SIGNAL_FUNC(do_exit), (gpointer) NONE);
    gtk_container_add( GTK_CONTAINER(hbox_end), button_ok);
    gtk_widget_show(button_ok);

    /* bouton quitter */
    button_quit=gtk_button_new_with_label("Quitter");
    gtk_signal_connect( GTK_OBJECT(button_quit), "clicked", GTK_SIGNAL_FUNC(question_exit), NULL);
    gtk_container_add( GTK_CONTAINER(hbox_end), button_quit);
    gtk_widget_show(button_quit);

    /* mise en place d'un hook pour assurer le retraage de la fentre
       principale de l'mulateur */
    gdk_window_add_filter(gdk_window_foreign_new(screen_win), retrace_callback, NULL);

    printf("ok\n");
}


void ControlPanel(void)
{
    /* affichage de la fentre principale et de ses lments */
    gtk_widget_show(window);
    
    /* boucle de gestion des vnements */
    gtk_main();
}
