/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : linux/main.c
 *  Version    : 1.4
 *  Cr par   : Eric Botcazou juillet 1999
 *  Modifi par: Eric Botcazou 15/12/2000
 *
 *  Boucle principale de l'mulateur
 */

#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <gtk/gtk.h>
#include <X11/Xlib.h>
#include <X11/Xresource.h>
#include <X11/Xutil.h>
#include "main.h"
#include "to7.h"
#include "linux/display.h"
#include "linux/graphic.h"
#include "linux/gui.h"
#include "linux/sound.h"


struct EmuTO thom={
    True,
    True,
    NONE
};

static int frame;           /* compteur de frame vido */
static volatile int tick;   /* compteur du timer */

#define OP_TABLE_SIZE 8
static XrmOptionDescRec op_table[OP_TABLE_SIZE]={
{"-help",	".help",		XrmoptionIsArg,		NULL},
{"-m",		".memo7",		XrmoptionSepArg,	NULL},
{"-fast",	".speed",		XrmoptionNoArg,		"fast"},
{"-nosound",	".sound",		XrmoptionNoArg,		"off"},
{"-geometry",	".geometry",		XrmoptionSepArg,	NULL},
{"-noshm",      ".shm",                 XrmoptionNoArg,         "off"},  
{"-display",	".display",		XrmoptionSepArg,	NULL},
{"-xrm",	NULL,			XrmoptionResArg,	NULL} 
};


static void Timer(int sigtype)
{
    tick++;
    signal(SIGALRM,Timer);
    (void) sigtype; /* pour viter un warning du compilateur */
}


/*
 *  RunTO7: boucle principale de l'mulateur
 */

static void RunTO7(void)
{
    const struct itimerval timer_value={ {0,1e6/TO7_FRAME_FREQ},
               				 {0,1e6/TO7_FRAME_FREQ} };

    if (!thom.sound_enabled)
    {
	signal(SIGALRM,Timer);
	setitimer(ITIMER_REAL, &timer_value, NULL);                     
    }

    frame=1;
    to7_ColdReset();

    do   /* boucle principale de l'mulateur */
    {
        thom.command=NONE;
	tick=frame;

        do  /* boucle d'mulation */
        {
            to7_DoFrame();

            if (need_border_refresh)
                RefreshBorder();

	    RefreshScreen();
            HandleEvents();

            if (thom.exact_speed)  /* synchronisation sur frquence relle */
            {
                if (thom.sound_enabled) 
		    PlaySoundBuffer();
                else if (frame==tick)
		    pause();  /* on attend le SIGALARM du timer */
            }

            frame++;
        }
	while (thom.command==NONE);  /* fin de la boucle d'mulation */
          
	if (thom.command==CONTROL_PANEL)
	    ControlPanel();

	if (thom.command==RESET)
	    to7_Reset();

	if (thom.command==COLD_RESET)
	    to7_ColdReset();

    }
    while (thom.command != QUIT);  /* fin de la boucle principale */
}


/*
 * GetUserInput: lit toutes les options spcifies par l'utilisateur dans
 *		 l'ordre de priorit dcroissante suivant:
 *		  -ligne de commande,
 *		  -ressource de l'cran mise en place par xrdb,
 *		  -fichier .Xdefaults dans le rpertoire utilisateur,
 *		 et les fusionne dans la base de donnes user_db.
 */
 
static XrmDatabase GetUserInput(int *argc, char *argv[])
{
    char display_name[FILENAME_LENGTH],file_name[FILENAME_LENGTH];
    char *str_type[20];
    XrmDatabase commandline_db=NULL, user_db;
    XrmValue value;
        
    XrmParseCommand(&commandline_db,op_table,OP_TABLE_SIZE,PROG_NAME,argc,argv);
            
    /* Demande d'aide ? */
    if (XrmGetResource(commandline_db,PROG_NAME".help",PROG_CLASS".Help",
       		                                           str_type, &value))
    {
        printf("Usage:\n");
        printf("           %s [options ...]\n",argv[0]);
        printf("o les options sont prises parmi les suivantes:\n");
        printf("    -help                   affiche cette aide\n");
        printf("    -m file.m7	     	    place la cartouche file.m7 dans le lecteur\n");		    		
        printf("    -fast                   active la pleine vitesse de l'mulateur\n");
        printf("    -nosound                supprime le son de l'mulateur\n");
	printf("    -geometry wxh+x+y       spcifie la gomtrie de la fentre\n");
	printf("                            (deux tailles possibles: 640x400 ou 320x200)\n");
	printf("    -noshm                  dsactive l'utilisation de l'extension MIT-SHM\n");
        printf("    -display displayname    spcifie le serveur X  utiliser\n");
        exit(EXIT_SUCCESS);
    }     
    
    /* On a besoin de display pour accder  la configuration utilisateur */
    if (XrmGetResource(commandline_db,PROG_NAME".display",PROG_CLASS".Display",
                                                        str_type, &value))
        strncpy(display_name, value.addr, value.size);
    else
	display_name[0]='\0';
    
    InitDisplay(display_name);

    if (XScreenResourceString(DefaultScreenOfDisplay(display)) != NULL)
        user_db=XrmGetStringDatabase(XScreenResourceString(DefaultScreenOfDisplay(display)));
    else if (XResourceManagerString(display) != NULL)
        user_db=XrmGetStringDatabase(XResourceManagerString(display));
    else
    {
        strcat(strcpy(file_name,getenv("HOME")),"./Xdefaults");
        user_db=XrmGetFileDatabase(file_name);
    }
    
    XrmMergeDatabases(commandline_db, &user_db);
    
    return user_db;
}


/*
 * SetParameters: fixe les paramtres de l'mulation,  partir des options de
 *		  l'utilisateur ou par dfaut.
 */
 
static void SetParameters(char memo_name[], int *x, int *y, int *user_flags, XrmDatabase user_db)
{
    char *str_type[20],buffer[20];
    int flags;
    XrmValue value;
    
    /* Fichier qui dcrit la cartouche place dans le lecteur */
    if (XrmGetResource(user_db,PROG_NAME".memo7",PROG_CLASS".Memo7",str_type, &value))
        strncpy(memo_name, value.addr, value.size); 

    /* Vitesse de l'mulation: relle (celle du TO7) ou rapide (celle du PC) */
    if (XrmGetResource(user_db,PROG_NAME".speed",PROG_CLASS".Speed",
                                        	           str_type, &value))
    {
        if (!strncmp(value.addr,"fast",value.size))
            thom.exact_speed=False;
        else if (strncmp(value.addr,"true",value.size))
        {
            fprintf(stderr,"%s: spcification de vitesse invalide\n",PROG_NAME);
            exit(EXIT_FAILURE);
        }
    }

    /* Activation de l'mulation sonore */
    if (XrmGetResource(user_db,PROG_NAME".sound",PROG_CLASS".Sound",
                                        	           str_type, &value))
    {
        if (!strncmp(value.addr,"off",value.size))
            thom.sound_enabled=False;
        else if (strncmp(value.addr,"on",value.size))
        {
            fprintf(stderr,"%s: spcification de l'mulation sonore invalide\n",PROG_NAME);
            exit(EXIT_FAILURE);
        }
    }

    /* Gomtrie de la fentre */
    screen_size=2;
    if (XrmGetResource(user_db,PROG_NAME".geometry",PROG_CLASS".Geometry",str_type, &value))
    {
        int w,h;
        
        strncpy(buffer, value.addr, value.size);
        flags=XParseGeometry(buffer, x, y, &w, &h);
         
	/* taille */
	if (flags&(WidthValue | HeightValue))
	{
	    if ((w==640) && (h==400))
		screen_size=2;
	    else if ((w==320) && (h==200))
		screen_size=1;
	    else
	    {
		fprintf(stderr,"%s: spcification de gomtrie invalide (voir %s -help)\n", PROG_NAME, PROG_NAME);
		exit(EXIT_FAILURE);
	    }
	}      
       
	/* position x */
        if (flags&XValue)
        {
            *user_flags|=USPosition;
            
            if (flags&XNegative)
                *x=DisplayWidth(display,screen)+*x-WINDOW_WIDTH;
        }
        
	/* position y */
        if (flags&YValue)
        {
            *user_flags|=USPosition;
            
            if (flags&YNegative)
                *y=DisplayHeight(display,screen)+*y-WINDOW_HEIGHT;
        }
    }
    
#ifdef DEBUG
    printf("Parameters: size=%d, width=%d, height=%d, x=%d, y=%d\n", screen_size, SCREEN_WIDTH, SCREEN_HEIGHT, *x, *y);
#endif
    
    /* Activation de l'extension MIT-SHM */
    if (XrmGetResource(user_db,PROG_NAME".shm",PROG_CLASS".Shm",str_type, &value))
    {
        if (!strncmp(value.addr,"off",value.size))
            mit_shm_enabled=False;
        else if (strncmp(value.addr,"on",value.size))
        {
            fprintf(stderr,"%s: spcification de l'extension MIT-SHM invalide\n",PROG_NAME);
            exit(EXIT_FAILURE);
        }
    }
}


static void ExitMessage(const char msg[])
{
    fprintf(stderr, "%s\n", msg);
    exit(EXIT_FAILURE);
}


int main(int argc,char *argv[])
{
    int x=0, y=0, user_flags=0;
    char version_name[]=TO7_VERSION" (Linux/X11)";
    char memo_name[FILENAME_LENGTH]="./memo7/basic.m7";
    XrmDatabase user_db;    

    /* Initialisation du serveur X */
    gtk_init(&argc, &argv);

    /* Initialisation du module de gestion des ressources */
    XrmInitialize();
    
    /* Mise en commun dans user_db toutes les commandes et options spcifies
       par l'utilisateur */
    user_db = GetUserInput(&argc, argv);   
    
    /* Mise en place des paramtres de l'mulation */
    SetParameters(memo_name, &x, &y, &user_flags, user_db);   
    XrmDestroyDatabase(user_db);    

    /* Affichage du message de bienvenue du programme */
    printf("Voici %s l'mulateur Thomson TO7-70.\n",version_name);
    printf("Copyright (C) 1996 Sylvain Huet, 1999 Eric Botcazou.\n\n");
    printf("Touches: [ESC] Panneau de contrle\n\n");
    
     /* Initialisation du TO7-70 */
    printf("Initialisation de l'mulateur...");
    fflush(stdout);

    if ( to7_Init() == TO7_ERROR )
        ExitMessage(to7_error_msg);
    
    printf("ok\n");

    /* Chargement de la cartouche */
    if (memo_name[0])
        to7_LoadMemo7(memo_name);
    
    /* Cration de la fentre principale de l'mulateur */
    InitWindow(argc, argv, x, y, user_flags);

     /* Initialisation de l'interface utilisateur */
    InitGUI(version_name);

    /* Initialisation des modules graphique et sonore */
    InitGraphic();
    InitSound();  
    
    /* Et c'est parti !!! */
    printf("Lancement de l'mulation...\n");
    RunTO7();
    
    /* Sortie de l'mulateur */
    printf("\nA bientt !\n");
    exit(EXIT_SUCCESS);
}
