/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : linux/sound.c
 *  Version    : 1.4
 *  Cr par   : Eric Botcazou aot 1999
 *  Modifi par: Eric Botcazou 15/12/2000
 *
 *  Gestion de l'mulation sonore du TO7-70
 */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include "main.h"
#include "to7.h"

#define SOUND_FREQ  25600

#define FRAG_EXP 9
#define SOUND_BUFFER_SIZE (1<<FRAG_EXP)
#define DEVNAME "/dev/dsp"

static int audio_fd;
static unsigned char sound_buffer[SOUND_BUFFER_SIZE];
static int last_index;
static unsigned char last_data;


static void PutSoundByte(unsigned long long int clock, unsigned char data)
{
    register int i;
    int index=(clock%TO7_CYCLES_PER_FRAME)*SOUND_FREQ/TO7_CPU_FREQ;

    /* Dans le cas o le nombre de cycles xcuts pendant une frame dpasse la valeur
       thorique, on bloque l'index  sa valeur maximale */
    if (index < last_index)
	index=SOUND_BUFFER_SIZE;

    for (i=last_index; i<index; i++)
        sound_buffer[i]=last_data;

    last_index=index;
    last_data=data;
}


int InitSound(void)
{
    int frag=0x10000+FRAG_EXP;
    int format=AFMT_U8;
    int stereo=0;
    int freq=SOUND_FREQ;

    to7_PutSoundByte=PutSoundByte;
    
    printf("Initialisation du son...");

    if ((audio_fd=open(DEVNAME, O_WRONLY))== -1)
    {
	perror(DEVNAME);
	return 0;
    }

    if (ioctl(audio_fd, SNDCTL_DSP_SETFRAGMENT, &frag) == -1)
    {
	perror("SNDCTL_DSP_SETFRAGMENT");
	close(audio_fd);
	return 0;
    }

    if (ioctl(audio_fd, SNDCTL_DSP_SETFMT, &format) == -1)
    {
	perror("SNDCTL_DSP_SETFMT");
	close(audio_fd);
	return 0;
    }

    if (format != AFMT_U8)
    {
	fprintf(stderr,"son 8-bit non support.\n");
	close(audio_fd);
	return 0;
    }

    if (ioctl(audio_fd, SNDCTL_DSP_STEREO, &stereo) == -1)
    {
	perror("SNDCTL_DSP_STEREO");
	close(audio_fd);
	return 0;
    }

    if (ioctl(audio_fd, SNDCTL_DSP_SPEED, &freq) == -1)
    {
	perror("SNDCTL_DSP_SPEED");
	close(audio_fd);
	return 0;
    }

    if (freq != SOUND_FREQ)
    {
	fprintf(stderr,"frquence requise non supporte.\n");
	close(audio_fd);
	return 0;
    }

    printf("ok\n");

#ifdef DEBUG
    {
	audio_buf_info info;

	ioctl(audio_fd, SNDCTL_DSP_GETOSPACE, &info);
	
	printf("frequency: %d Hz\n", freq); 
	printf("fragments: %d\n", info.fragstotal);
	printf("fragment size: %d bytes\n", info.fragsize);
    }
#endif

    return 1;
}


void PlaySoundBuffer(void)
{
    register int i;

     /* on remplit la fin du buffer avec le dernier byte dpos */
    for (i=last_index; i<SOUND_BUFFER_SIZE; i++)
        sound_buffer[i]=last_data;

    last_index=0;

    write(audio_fd, sound_buffer, SOUND_BUFFER_SIZE);
}
