/*
 *  Module d'mulation des micro-circuits Motorola MC68xx:
 *    - microprocesseur MC6809E
 *    - PIA MC6846
 *    - PIA MC6821
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : mc68xx/mc6821.c
 *  Version    : 3.2
 *  Cr par   : Eric Botcazou
 *  Modifi par: Eric Botcazou 17/06/2000
 *
 *  Emulation du PIA Motorola MC6821
 */

#ifdef DEBUG
#include "mc68xx/mc6821.h"


/*
 * mc6821_WriteCommand: dpose une valeur dans le registre de commande du port
 */

void mc6821_WriteCommand(struct MC6821_PORT *port, int val)
{
    if (val&0x30)  /* CP2 en sortie */
        port->cr = (port->cr&0xC0) | (val&0x3F);
    else
        port->cr = (port->cr&0xC8) | (val&0x37);
}


/*
 * mc6821_ReadCommand: lit une valeur dans le registre de commande du port
 */

int mc6821_ReadCommand(struct MC6821_PORT *port)
{
    return port->cr;
}


/*
 * mc6821_WriteData: crit une valeur sur le port
 */

void mc6821_WriteData(struct MC6821_PORT *port, int val)
{
    int mask;

    if (port->cr&4)
	port->odr = val;
    else
        port->ddr = val;

    mask = port->ddr & port->w_mask;
    port->pdr = (port->pdr&(mask^0xFF)) | (port->odr&mask);
}


/*
 * mc6821_ReadData: retourne la valeur du registre de donnes du port
 */

int mc6821_ReadData(struct MC6821_PORT *port)
{
    if (port->cr&4)
	return (port->pdr&(port->ddr^0xFF)) | (port->odr&port->ddr);
    else
	return port->ddr;
}


/*
 * mc6821_Init: initialise le port spcifi
 */

void mc6821_Init(struct MC6821_PORT *port, int cr, int pdr, int w_mask)
{
    port->cr     = cr;
    port->ddr    = 0;
    port->pdr    = pdr;
    port->odr    = 0;
    port->w_mask = w_mask;
}

#endif

